@@ -300,7 +300,8 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
               {
                 index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
                 SetPixelIndex(indexes+x+bit,index);
-                SetPixelRGBO(q,image->colormap+(ssize_t) index);
+                if (index < image->colors)
+                  SetPixelRGBO(q,image->colormap+(ssize_t) index);
                 q++;
               }
             p++;
@@ -311,7 +312,8 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
               {
                 index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
                 SetPixelIndex(indexes+x+bit,index);
-                SetPixelRGBO(q,image->colormap+(ssize_t) index);
+                if (index < image->colors)
+                  SetPixelRGBO(q,image->colormap+(ssize_t) index);
                 q++;
               }
             p++;
@@ -327,19 +329,23 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
         {
             index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
             SetPixelIndex(indexes+x,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             q++;
             index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
             SetPixelIndex(indexes+x,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             q++;
             index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
             SetPixelIndex(indexes+x,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             q++;
             index=ConstrainColormapIndex(image,(*p) & 0x3);
             SetPixelIndex(indexes+x+1,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             q++;
             p++;
         }
@@ -347,19 +353,22 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
           {
             index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
             SetPixelIndex(indexes+x,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             q++;
             if ((image->columns % 4) > 1)
               {
                 index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
                 SetPixelIndex(indexes+x,index);
-                SetPixelRGBO(q,image->colormap+(ssize_t) index);
+                if (index < image->colors)
+                  SetPixelRGBO(q,image->colormap+(ssize_t) index);
                 q++;
                 if ((image->columns % 4) > 2)
                   {
                     index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
                     SetPixelIndex(indexes+x,index);
-                    SetPixelRGBO(q,image->colormap+(ssize_t) index);
+                    if (index < image->colors)
+                      SetPixelRGBO(q,image->colormap+(ssize_t) index);
                     q++;
                   }
               }
@@ -374,11 +383,13 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
           {
             index=ConstrainColormapIndex(image,(*p >> 4) & 0x0f);
             SetPixelIndex(indexes+x,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             q++;
             index=ConstrainColormapIndex(image,(*p) & 0x0f);
             SetPixelIndex(indexes+x+1,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             p++;
             q++;
           }
@@ -386,7 +397,8 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
           {
             index=ConstrainColormapIndex(image,(*p >> 4) & 0x0f);
             SetPixelIndex(indexes+x,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             p++;
             q++;
           }
@@ -398,7 +410,8 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
           {
             index=ConstrainColormapIndex(image,*p);
             SetPixelIndex(indexes+x,index);
-            SetPixelRGBO(q,image->colormap+(ssize_t) index);
+            if (index < image->colors)
+              SetPixelRGBO(q,image->colormap+(ssize_t) index);
             p++;
             q++;
           }
@@ -456,6 +469,7 @@ static int UnpackWPGRaster(Image *image,int bpp)
   BImgBuff=(unsigned char *) AcquireQuantumMemory((size_t) ldblk,
     8*sizeof(*BImgBuff));
   if(BImgBuff==NULL) return(-2);
+  (void) memset(BImgBuff,0,(size_t) ldblk*8*sizeof(*BImgBuff));
 
   while(y<(ssize_t) image->rows)
   {
@@ -487,14 +501,19 @@ static int UnpackWPGRaster(Image *image,int bpp)
           {
             for(i=0;i < (int) RunCount;i++)
               {
-                bbuf=ReadBlobByte(image);
-                InsertByte(bbuf);
+                c=ReadBlobByte(image);
+                if (c < 0)
+                  break;
+                InsertByte(c);
               }
           }
         else {  /* repeat previous line runcount* */
           c=ReadBlobByte(image);
-          if (c < 0)
-            break;
+          if (c == EOF)
+            {
+              BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+              return(-7);
+            }
           RunCount=(unsigned char) c;
           if(x) {    /* attempt to duplicate row from x position: */
             /* I do not know what to do here */
@@ -575,6 +594,7 @@ static int UnpackWPG2Raster(Image *image,int bpp)
     sizeof(*BImgBuff));
   if(BImgBuff==NULL)
     return(-2);
+  (void) memset(BImgBuff,0,ldblk*sizeof(*BImgBuff));
 
   while( y< image->rows)
     {
@@ -596,8 +616,9 @@ static int UnpackWPG2Raster(Image *image,int bpp)
             }
           break;
         case 0x7E:
-          (void) FormatLocaleFile(stderr,
-            "\nUnsupported WPG token XOR, please report!");
+          if (y == 0)
+            (void) FormatLocaleFile(stderr,
+              "\nUnsupported WPG token XOR, please report!");
           XorMe=!XorMe;
           break;
         case 0x7F:
@@ -758,6 +779,9 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   Image
     *image2;
 
+  MagickBooleanType
+    status;
+
   unsigned char
     magick[2*MaxTextExtent];
 
@@ -768,6 +792,7 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
     return(image);
   clone_info->blob=(void *) NULL;
   clone_info->length=0;
+  status=MagickFalse;
 
   /* Obtain temporary file */
   (void) AcquireUniqueFilename(postscript_file);
@@ -776,7 +801,11 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
     goto FINISH;
 
   /* Copy postscript to temporary file */
-  (void) SeekBlob(image,PS_Offset,SEEK_SET);
+  if (SeekBlob(image,PS_Offset,SEEK_SET) != PS_Offset)
+    {
+      DestroyImageInfo(clone_info);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
   count=ReadBlob(image, 2*MaxTextExtent, magick);
   if (count < 1)
     {
@@ -784,7 +813,11 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
 
-  (void) SeekBlob(image,PS_Offset,SEEK_SET);
+  if (SeekBlob(image,PS_Offset,SEEK_SET) != PS_Offset)
+    {
+      DestroyImageInfo(clone_info);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
   while (PS_Size-- > 0)
   {
     c=ReadBlobByte(image);
@@ -800,12 +833,17 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   /*     printf("Detected:%s  \n",magic_info->name); */
   if(exception->severity != UndefinedException) goto FINISH_UNL;
   if(magic_info->name == (char *) NULL) goto FINISH_UNL;
+  if (LocaleCompare(magic_info->name,"WPG") == 0)
+    goto FINISH_UNL;
 
   (void) strncpy(clone_info->magick,magic_info->name,MaxTextExtent-1);
+  if (LocaleCompare(image_info->magick,clone_info->magick) == 0)
+    (void) strcpy(clone_info->magick,"PS");
 
     /* Read nested image */
   /*FormatString(clone_info->filename,"%s:%s",magic_info->name,postscript_file);*/
-  FormatLocaleString(clone_info->filename,MaxTextExtent,"%s",postscript_file);
+  FormatLocaleString(clone_info->filename,MagickPathExtent,"%.1024s:%.1024s",
+    clone_info->magick,postscript_file);
   image2=ReadImage(clone_info,exception);
 
   if (!image2)
@@ -813,7 +851,7 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   if(exception->severity>=ErrorException)
   {
     CloseBlob(image2);
-    DestroyImageList(image2);  
+    DestroyImageList(image2);
     goto FINISH_UNL;
   }
 
@@ -831,11 +869,10 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
       (void) CopyMagickString(p->magick_filename,image->magick_filename,
         MagickPathExtent);
       (void) CopyMagickString(p->magick,image->magick,MagickPathExtent);
-      DestroyBlob(p);      
       if ((p->rows == 0) || (p->columns == 0))
         {
           DeleteImageFromList(&p);
-          if (p == (Image *) NULL) 
+          if (p == (Image *) NULL)
           {
             image2=(Image *) NULL;
             goto FINISH_UNL;
@@ -843,6 +880,7 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
         }
       else
         {
+          DestroyBlob(p);
           p->blob=ReferenceBlob(image->blob);
           p=p->next;
         }
@@ -857,12 +895,15 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
 
   AppendImageToList(&image,image2);
   while (image->next != NULL)
-    image=image->next;    
+    image=image->next;
+  status=MagickTrue;
 
  FINISH_UNL:
   (void) RelinquishUniqueFileResource(postscript_file);
  FINISH:
   DestroyImageInfo(clone_info);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(image);
 }
 
@@ -1058,6 +1099,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   image->columns = 1;
   image->rows = 1;
   image->colors = 0;
+  (void) ResetImagePixels(image,exception);
   bpp=0;
   BitmapHeader2.RotAngle=0;
   Rec2.RecordLength = 0;
@@ -1067,7 +1109,8 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     case 1:     /* WPG level 1 */
       while(!EOFBlob(image)) /* object parser loop */
         {
-          (void) SeekBlob(image,Header.DataOffset,SEEK_SET);
+          if (SeekBlob(image,Header.DataOffset,SEEK_SET) != Header.DataOffset)
+            break;
           if(EOFBlob(image))
             break;
 
@@ -1125,6 +1168,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     ReadBlobByte(image));
                   image->colormap[i].blue=ScaleCharToQuantum((unsigned char)
                     ReadBlobByte(image));
+                  image->colormap[i].opacity=OpaqueOpacity;
                 }
               break;
 
@@ -1174,9 +1218,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               status=SetImageExtent(image,image->columns,image->rows);
               if (status == MagickFalse)
                 break;
+              (void) ResetImagePixels(image,exception);
               if ((image->storage_class != PseudoClass) && (bpp < 24))
                 {
                   image->colors=one << bpp;
+                  if (image->colors > GetBlobSize(image))
+                    ThrowReaderException(CorruptImageError,
+                      "InsufficientImageDataInFile");
                   if (!AcquireImageColormap(image,image->colors))
                     {
                     NoMemory:
@@ -1189,6 +1237,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                       image->colormap[i].red=ScaleCharToQuantum(WPG1_Palette[i].Red);
                       image->colormap[i].green=ScaleCharToQuantum(WPG1_Palette[i].Green);
                       image->colormap[i].blue=ScaleCharToQuantum(WPG1_Palette[i].Blue);
+                      image->colormap[i].opacity=OpaqueOpacity;
                     }
                 }
               else
@@ -1200,7 +1249,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                         sizeof(*image->colormap));
                 }
 
-              if (bpp == 1)
+              if ((bpp == 1) && (image->colors > 1))
                 {
                   if(image->colormap[0].red==0 &&
                      image->colormap[0].green==0 &&
@@ -1212,15 +1261,17 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                       image->colormap[1].red =
                         image->colormap[1].green =
                         image->colormap[1].blue = QuantumRange;
+                      image->colormap[1].opacity=OpaqueOpacity;
                     }
                 }
 
-              if(UnpackWPGRaster(image,bpp) < 0)
-                /* The raster cannot be unpacked */
-                {
-                DecompressionFailed:
-                  ThrowReaderException(CoderError,"UnableToDecompressImage");
-                    }
+              if(!image_info->ping)
+                if(UnpackWPGRaster(image,bpp) < 0)
+                  /* The raster cannot be unpacked */
+                  {
+                  DecompressionFailed:
+                    ThrowReaderException(CoderError,"UnableToDecompressImage");
+                      }
 
               if(Rec.RecType==0x14 && BitmapHeader2.RotAngle!=0 && !image_info->ping)
                 {
@@ -1269,7 +1320,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if (image->next == (Image *) NULL)
                 goto Finish;
               image=SyncNextImageInList(image);
-              image->columns=image->rows=1;
+              image->columns=image->rows=0;
               image->colors=0;
               break;
 
@@ -1293,7 +1344,8 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
       StartWPG.PosSizePrecision = 0;
       while(!EOFBlob(image)) /* object parser loop */
         {
-          (void) SeekBlob(image,Header.DataOffset,SEEK_SET);
+          if (SeekBlob(image,Header.DataOffset,SEEK_SET) != Header.DataOffset)
+            break;
           if(EOFBlob(image))
             break;
 
@@ -1323,6 +1375,8 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if ((WPG_Palette.NumOfEntries-WPG_Palette.StartIndex) >
                   (Rec2.RecordLength-2-2) / 3)
                 ThrowReaderException(CorruptImageError,"InvalidColormapIndex");
+              if (WPG_Palette.StartIndex >= WPG_Palette.NumOfEntries)
+                ThrowReaderException(CorruptImageError,"InvalidColormapIndex");
               image->colors=WPG_Palette.NumOfEntries;
               if (AcquireImageColormap(image,image->colors) == MagickFalse)
                 ThrowReaderException(ResourceLimitError,
@@ -1336,6 +1390,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     ReadBlobByte(image));
                   image->colormap[i].blue=ScaleCharToQuantum((char)
                     ReadBlobByte(image));
+                  image->colormap[i].opacity=OpaqueOpacity;
                   (void) ReadBlobByte(image);   /*Opacity??*/
                 }
               break;
@@ -1371,7 +1426,11 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                 }
               image->columns=Bitmap2Header1.Width;
               image->rows=Bitmap2Header1.Height;
+              if (image_info->ping != MagickFalse)
+                return(image);
               status=SetImageExtent(image,image->columns,image->rows);
+              if (status != MagickFalse)
+                status=ResetImagePixels(image,exception);
               if (status == MagickFalse)
                 break;
               if ((image->colors == 0) && (bpp != 24))
@@ -1403,20 +1462,20 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                       ldblk+1,sizeof(*BImgBuff));
                     if (BImgBuff == (unsigned char *) NULL)
                       goto NoMemory;
-
-                    for (i=0; i< (ssize_t) image->rows; i++)
+                    for (i=0; i < (ssize_t) image->rows; i++)
                     {
-                      (void) ReadBlob(image,ldblk,BImgBuff);
-                      if (InsertRow(BImgBuff,i,image,bpp) == MagickFalse)
-                        {
-                          if(BImgBuff)
-                            BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
-                          goto DecompressionFailed;
-                        }
-                    }
+                      ssize_t
+                        count;
 
-                    if(BImgBuff)
-                      BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+                      count=ReadBlob(image,(size_t) ldblk,BImgBuff);
+                      if (count != ldblk)
+                        break;
+                      if (InsertRow(BImgBuff,i,image,bpp) == MagickFalse)
+                        break;
+                    }
+                    BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+                    if (i < (ssize_t) image->rows)
+                      goto DecompressionFailed;
                     break;
                   }
                 case 1:    /*RLE for WPG2 */
@@ -1468,7 +1527,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if (image->next == (Image *) NULL)
                 goto Finish;
               image=SyncNextImageInList(image);
-              image->columns=image->rows=1;
+              image->columns=image->rows=0;
               image->colors=0;
               break;
 
