@@ -622,6 +622,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
 
   size_t
     height,
+    imageListLength,
     quantum,
     width;
 
@@ -681,6 +682,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         return(status);
     }
   scene=0;
+  imageListLength=GetImageListLength(image);
   do
   {
     /*
@@ -861,8 +863,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
